
namespace pigOnTheScreen {
    string currentPigImagePath = "";
    UI::Texture@ currentPigImage;
    vec2 currentPigMovement;
    string pigsFolder = IO::FromStorageFolder("pig-images");
    vec2 pigPos(50, 50);

    void init() {
        downloadPigImages();
        startnew(pigBrain);
    }

    void pigBrain() {
        pigNewAction();
        while (true) {
            if (Relax) {
                sleep(2000);
                pigIdleAction();
            }
            sleep(2000);
            pigNewAction();
        }
    }

    void pigIdleAction() {
        currentPigImagePath = pigsFolder + "/" + "pig-idle.png";
        currentPigMovement = vec2(0, 0);
        IO::File iof(currentPigImagePath, IO::FileMode::Read);
        @currentPigImage = UI::LoadTexture(iof.Read(iof.Size()));
        iof.Close();
    }

    void pigNewAction() {
        int[][] possiblePigMovement = {{0, 0}, {0, 1}, {-1, 0}, {1, 0}, {0, -1}};
        string[] possiblePigImages = {"pig-idle.png", "pig-walk-down.png", "pig-walk-left.png", "pig-walk-right.png", "pig-walk-up.png"};

        uint nextAction = Math::Rand(0, Math::Min(possiblePigImages.Length, possiblePigMovement.Length));
        currentPigImagePath = pigsFolder + "/" + possiblePigImages[nextAction];
        int[] PigMovement = possiblePigMovement[nextAction];
        currentPigMovement = vec2(PigMovement[0], PigMovement[1]);
        IO::File iof(currentPigImagePath, IO::FileMode::Read);
        @currentPigImage = UI::LoadTexture(iof.Read(iof.Size()));
        iof.Close();
    }

    void downloadPigImages() {
        if (!IO::FolderExists(pigsFolder)) IO::CreateFolder(pigsFolder);
        if (pigsFolder.EndsWith("/")) pigsFolder = pigsFolder.SubStr(0, pigsFolder.Length-1);
        Net::HttpRequest@ req = Net::HttpGet("https://openplanet.dev/plugin/pp/config/pig-on-the-screen");
        while (!req.Finished()) yield();

        Json::Value json = req.Json();

        if (json.GetType() != Json::Type::Array) return;

        for (uint i = 0; i < json.Length; i++) {
            if(json[i].GetType() != Json::Type::Array || json[i].Length < 2 || json[i][0].GetType() != Json::Type::String || json[i][1].GetType() != Json::Type::String) continue;
            downloadOneImage(string(json[i][1]), string(json[i][0]));
        }
    }

    void downloadOneImage(const string &in url, const string &in fileNameIn) {
        string fileName = fileNameIn.Replace("/", "").Replace("\\", "");

        Net::HttpRequest@ req = Net::HttpGet(url);
        while(!req.Finished()) yield();
        if (req.ResponseCode() != 200) {
            trace("[PP] bad response code. Check network log!");
            trace(url);
            return;
        }

        if (IO::FileExists(pigsFolder+"/"+fileName)) return;

        trace("[PP] saving " + fileName);
        IO::File file(pigsFolder+"/"+fileName, IO::FileMode::Write);
        file.Write(req.Buffer());
        file.Close();
    }

    void render() {
        if (!pigOnTheScreenOn || currentPigImage is null) return;

        UI::DrawList@ dl = UI::GetForegroundDrawList();
        vec2 screen(Draw::GetWidth(), Draw::GetHeight());
        dl.AddImage(currentPigImage, pigPos);
        pigPos += currentPigMovement;
        if (pigPos.x < 10) pigPos.x = 10;
        if (pigPos.x > Draw::GetWidth()-10) pigPos.x = Draw::GetWidth()-10;
        if (pigPos.y < 10) pigPos.y = 10;
        if (pigPos.y > Draw::GetHeight()-10) pigPos.y = Draw::GetHeight()-10;
    }
}
